@@ -184,10 +184,7 @@ namespace zyppng {
 
       if ( urlScheme == "https" )
       {
-        // restrict following of redirections from https to https only
-        // but be less restrictive for d.o.o
-        bool allowHttp = ( _url.getHost() == "download.opensuse.org" );
-        if ( :: internal::setCurlRedirProtocols ( _easyHandle, allowHttp ) != CURLE_OK ) {
+        if ( :: internal::setCurlRedirProtocols ( _easyHandle ) != CURLE_OK ) {
           ZYPP_THROW( zypp::media::MediaCurlSetOptException( _url, _errorBuf.data() ) );
         }
 
@@ -442,6 +439,7 @@ namespace zyppng {
       }
       if ( !setupHandle(errBuf))
         return false;
+
       return true;
     }
     errBuf = "Request has no more work";
@@ -722,6 +720,21 @@ namespace zyppng {
     return written;
   }
 
+  void NetworkRequestPrivate::notifyErrorCodeChanged()
+  {
+    auto rmode = std::get_if<NetworkRequestPrivate::running_t>( &_runningMode );
+    if ( !rmode || !rmode->_partialHelper || !rmode->_partialHelper->hasError() )
+      return;
+
+    // oldest cached result wins
+    if ( rmode->_cachedResult )
+      return;
+
+    auto lastErr = NetworkRequestErrorPrivate::customError( rmode->_partialHelper->lastError() , std::string(rmode->_partialHelper->lastErrorMessage()) );
+    MIL_MEDIA << _easyHandle << " Multipart handler announced error code " << lastErr.toString () <<  std::endl;
+    rmode->_cachedResult = lastErr;
+  }
+
   ZYPP_IMPL_PRIVATE(NetworkRequest)
 
   NetworkRequest::NetworkRequest(zyppng::Url url, zypp::filesystem::Pathname targetFile, zyppng::NetworkRequest::FileMode fMode)
